#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 position_light_space;
} vert;

vec4 color;
layout(location = 0) out vec4 FragColor;
layout(location = 1) out vec4 brightColor;

uniform vec3 camera_world;
uniform vec3 materialCoefficients = vec3(0.1, 0.7, 0.1); // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha = 1.0;
uniform vec3 diffuseColor;
uniform float scale = 1.0;
// for shadows
uniform sampler2D shadowMap;

uniform sampler2D diffuseTexture;

vec3 color1 = vec3(1.0, 0.0, 0.0);
vec3 color2 = vec3(0.0, 1.0, 0.0);

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}
// https://learnopengl.com/Advanced-Lighting/Shadows/Shadow-Mapping
float ShadowCalculation(vec4 position_light_space_)
{
	// perform perspective divide
	vec3 projCoords = position_light_space_.xyz / position_light_space_.w;
	// transform to [0,1] range, [u,v] 
	projCoords = projCoords * 0.5 + 0.5;
	// get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
	float closestDepth = texture(shadowMap, projCoords.xy).r;
	// get depth of current fragment from light's perspective
	float currentDepth = projCoords.z;
	vec3 normal = normalize(vert.normal_world);
	// check whether current frag pos is in shadow
	float bias = max(0.05 * (1.0 - dot(vert.normal_world, dirL.direction)), 0.001);

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
			float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r;
			shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;
		}
	}
	shadow /= 9.0;
	
	if(projCoords.z > 1.0)
		shadow = 0.0;
	
	return shadow;
}

void main() {	
	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	u_Scale = 50.0 * scale;

	float shadow = ShadowCalculation(vert.position_light_space);

	color = vec4(checkerboard(vert.uv) * materialCoefficients.x, 1); // ambient

	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0)) * (1.0 - shadow);
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);

	FragColor = color;
	// extract bright color
	// check color is higher than some thershold
	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
	if(brightness > 1.0){
		brightColor = vec4(color.rgb, 1.0);
		//color = vec4(0); debug
		}
	else{
		brightColor = vec4(0.0, 0.0, 0.0, 1.0);
		//color = vec4(1); debug
		}

}